/*
 * board-omap3tao3530.c (TechNexion TAO-3530 Board)
 * Maintainer:
 *        Edward Lin <linuxfae@technexion.com>
 *
 * Modified from Beagle board and OMAP3 EVM
 *          from mach-omap2/board-omap3beagle.c and mach-omap2/board-omap3evm.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/io.h>
#include <linux/leds.h>
#include <linux/gpio.h>
#include <linux/input.h>
#include <linux/gpio_keys.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>

#include <linux/regulator/machine.h>
#include <linux/i2c/twl.h>
#include <linux/backlight.h>

#include <mach/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <plat/board.h>
#include <plat/usb.h>
#include <linux/usb/otg.h>
#include <plat/common.h>
#include <plat/gpmc.h>
#include <plat/nand.h>
#include <plat/mux.h>
#include <plat/display.h>

#include "mmc-twl4030.h"

#include "mux.h"
#include "sdram-micron-mt46h32m32lf-6.h"
#include <plat/mmc.h> 

#include "pm.h"
#include "prm-regbits-34xx.h"
#include "omap3-opp.h"

#if defined(CONFIG_VIDEO_TVP514X) || defined(CONFIG_VIDEO_TVP514X_MODULE)
#include <media/tvp514x-int.h>
extern struct tvp514x_platform_data tvp5146_pdata;
#endif

#if defined(CONFIG_TAO3530_THUNDER_CARRIER)
	#define TAO3530_LCD_ENABLED
	#define TAO3530_TOUCHSCREEN_ENABLED
	#define TAO3530_GSENSOR_ENABLED
	#define TAO3530_DVI_ENABLED
	#define TAO3530_NAME_STRING "OMAP3-TAO3530 : Thunder Pack"
#endif

#if defined(CONFIG_TAO3530_TSUNAMI_CARRIER)
	#define TAO3530_LCD_ENABLED
	#define TAO3530_TOUCHSCREEN_ENABLED
	#define TAO3530_SMSC9220_ENABLED
	#define TAO3530_TV_OUTPUT_ENABLED
	#define TAO3530_VIDEO_INPUT_ENABLED
	#define TAO3530_DVI_ENABLED
	#define TAO3530_SD_WRITE_PROTECT_OFF
        #define TAO3530_NAME_STRING "OMAP3-TAO3530 : Tsunami Pack"
#endif

#if defined(CONFIG_TAO3530_INFERNO_CARRIER)
	#define TAO3530_TV_OUTPUT_ENABLED
	#define TAO3530_DVI_ENABLED
        #define TAO3530_NAME_STRING "OMAP3-TAO3530 : Inferno Pack"
#endif

#if defined(CONFIG_LIBERTAS)||defined(CONFIG_LIBERTAS_MODULE)
	#define TAO3530_SDIO_WIFI
	#define GPIO_WIFI_NRESET	157
#endif

/*Finetune Electronic Characters for each TouchScreen*/
#if defined(CONFIG_PANEL_LG_LB043WQ2)
	#define TS_PLATE_OHMS		50
	#define TS_DEBOUNCE_MAX		70
	#define TS_DEBOUNCE_TOL 	30
	#define TS_DEBOUNCE_REP		1
	#define TS_SETTLE_DELAY		75
#elif defined(CONFIG_PANEL_INNOLUX_AT070TN93)
	#define TS_PLATE_OHMS           150
	#define TS_DEBOUNCE_MAX         70
	#define TS_DEBOUNCE_TOL 	30
	#define TS_DEBOUNCE_REP		1
	#define TS_SETTLE_DELAY         150
#else
        #define TS_PLATE_OHMS           80
        #define TS_DEBOUNCE_MAX         10
	#define TS_DEBOUNCE_TOL 	3
	#define TS_DEBOUNCE_REP		1
        #define TS_SETTLE_DELAY         100
#endif

#ifdef CONFIG_PM
static struct omap_opp * _omap35x_mpu_rate_table	= omap35x_mpu_rate_table;
static struct omap_opp * _omap37x_mpu_rate_table	= omap37x_mpu_rate_table;
static struct omap_opp * _omap35x_dsp_rate_table	= omap35x_dsp_rate_table;
static struct omap_opp * _omap37x_dsp_rate_table	= omap37x_dsp_rate_table;
static struct omap_opp * _omap35x_l3_rate_table		= omap35x_l3_rate_table;
static struct omap_opp * _omap37x_l3_rate_table		= omap37x_l3_rate_table;
#else	/* CONFIG_PM */
static struct omap_opp * _omap35x_mpu_rate_table	= NULL;
static struct omap_opp * _omap37x_mpu_rate_table	= NULL;
static struct omap_opp * _omap35x_dsp_rate_table	= NULL;
static struct omap_opp * _omap37x_dsp_rate_table	= NULL;
static struct omap_opp * _omap35x_l3_rate_table		= NULL;
static struct omap_opp * _omap37x_l3_rate_table		= NULL;
#endif	/* CONFIG_PM */

#define GPMC_CS0_BASE  0x60
#define GPMC_CS_SIZE   0x30
#define NAND_BLOCK_SIZE	SZ_128K

#if defined(TAO3530_SMSC9220_ENABLED) && \
	(defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE))

#include <linux/smsc911x.h>

#define OMAP3TAO3530_SMSC911X_GPIO_IRQ	143
#define OMAP3TAO3530_SMSC911X_GPIO_RESET 142
#define OMAP3TAO3530_SMSC911X_CS	5

static struct resource omap3tao3530_smsc911x_resources[] = {
	{
		.name	= "smsc911x-memory",
		.flags	= IORESOURCE_MEM,
	},
	{
		.flags	=  (IORESOURCE_IRQ | IRQF_TRIGGER_LOW),
	},
};

static struct smsc911x_platform_config smsc911x_config = {
	.irq_polarity	= SMSC911X_IRQ_POLARITY_ACTIVE_LOW,
	.irq_type	= SMSC911X_IRQ_TYPE_OPEN_DRAIN,
	.flags		= (SMSC911X_USE_32BIT | SMSC911X_SAVE_MAC_ADDRESS),
	.phy_interface	= PHY_INTERFACE_MODE_MII,
};

static struct platform_device omap3tao3530_smsc911x_device = {
	.name		= "smsc911x",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(omap3tao3530_smsc911x_resources),
	.resource	= &omap3tao3530_smsc911x_resources,
	.dev		= {
		.platform_data = &smsc911x_config,
	},
};

static inline void __init omap3tao3530_init_smsc911x(void)
{
	unsigned long cs_mem_base;

	if (gpmc_cs_request(OMAP3TAO3530_SMSC911X_CS, SZ_16M, &cs_mem_base) < 0) {
		printk(KERN_ERR "Failed request for GPMC mem for smsc911x\n");
		return;
	}

	omap3tao3530_smsc911x_resources[0].start = cs_mem_base + 0x0;
	omap3tao3530_smsc911x_resources[0].end   = cs_mem_base + SZ_16K;//0xFF;

	omap_mux_init_gpio(OMAP3TAO3530_SMSC911X_GPIO_IRQ, OMAP_PIN_INPUT_PULLUP|OMAP_MUX_MODE4);

	if ((gpio_request(OMAP3TAO3530_SMSC911X_GPIO_IRQ, "SMSC911X IRQ") == 0) &&
	    (gpio_direction_input(OMAP3TAO3530_SMSC911X_GPIO_IRQ) == 0)) {
		gpio_export(OMAP3TAO3530_SMSC911X_GPIO_IRQ, 0);
	} else {
		printk(KERN_ERR "could not obtain gpio for SMSC911X IRQ\n");
		return;
	}

	omap3tao3530_smsc911x_resources[1].start = OMAP_GPIO_IRQ(OMAP3TAO3530_SMSC911X_GPIO_IRQ);
	omap3tao3530_smsc911x_resources[1].end  = OMAP_GPIO_IRQ(OMAP3TAO3530_SMSC911X_GPIO_IRQ);
#if 1
	omap_mux_init_gpio(OMAP3TAO3530_SMSC911X_GPIO_RESET, OMAP_PIN_INPUT_PULLUP|OMAP_MUX_MODE4);

	if (gpio_request(OMAP3TAO3530_SMSC911X_GPIO_RESET, "smsc911x reset") < 0){
                printk(KERN_ERR "can't get smsc911x reset GPIO\n");
                return;
        }

        gpio_direction_output(OMAP3TAO3530_SMSC911X_GPIO_RESET,0);
        mdelay(1);
        gpio_direction_output(OMAP3TAO3530_SMSC911X_GPIO_RESET,1);
#endif
	platform_device_register(&omap3tao3530_smsc911x_device);
}

#else
static inline void __init omap3tao3530_init_smsc911x(void) { return; }
#endif

static int lcd_enabled;
static int dvi_enabled;
static int tv_enabled;

#if defined(TAO3530_LCD_ENABLED)
/*
 * OMAP3TAO3530 LCD Panel control signals
 */
#define TAO3530_LCD_PANEL_ENVDD 138 
#define TAO3530_LCD_PON 	139
#define TAO3530_LCD_ENBKL 	198 /* GPIO 6 on TWL4030 */

#define TWL_INTBR_GPBR1 0x0c
#define TWL_INTBR_PMBR1 0x0d
#define TWL_PWM_ON    	0x00
#define TWL_PWM_OFF  	0x01

#define DEFAULT_BACKLIGHT_LEVEL 50
static int tao3530_backlight_level=DEFAULT_BACKLIGHT_LEVEL;

static void tao3530_set_backlight_intensity(int intensity)
{
        unsigned char backlight_intensity;

        if (intensity > 100)
                return;

	tao3530_backlight_level = intensity;

        /*
        * For 0,1%   of duty cycle (PWM_ON, PWM_OFF) = (0x0f, 0x10)
        * For 11 %   of duty cycle (PWM_ON, PWM_OFF) = (0x01, 0x10)
        * For 48%    of duty cycle (PWM_ON, PWM_OFF) = (0x01, 0x3f)
        * For 73.4%  of duty cycle (PWM_ON, PWM_OFF) = (0x01, 0x5f)
        * For 85.92% of duty cycle (PWM_ON, PWM_OFF) = (0x01, 0x6f)
        * For 98%    of duty cycle (PWM_ON, PWM_OFF) = (0x01, 0x7f)
        */

#if defined(CONFIG_LVDS_18BIT)
        /*Character of LVDS Interface control:
         *PWM0/TWL_GPIO6 set as GPIO to control backlight On/Off
         *PWM1/TWL_GPIO7 set as PWM to control backlight Dimming*/

        /*PWM0 Enable set to 0x04, Enable PWM1 set to 0x30, Both set to 0x34 */
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x30, TWL_INTBR_PMBR1);
        /*PWM0 Enable set to 0x05, PWM1 Enable set to 0x0a, Both set to 0x0f */
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x0a, TWL_INTBR_GPBR1);

	backlight_intensity =
	#if defined(CONFIG_PANEL_LITEMAX_DLF1095)/*Higher Duty Cycle with Lower Luminance*/
		((125 * (100 - intensity)) / 100) + 2;
	#else
		((125 * (intensity)) / 100) + 2;
	#endif

        twl_i2c_write_u8(TWL4030_MODULE_PWM1, 0x1, TWL_PWM_ON);
        twl_i2c_write_u8(TWL4030_MODULE_PWM1, backlight_intensity , TWL_PWM_OFF);

#else
        /*Character of Other LCD Interface control:
         *PWM0/TWL_GPIO6 set as PWM to control backlight Dimming*/

        //PWM0 Enable set to 0x04, Enable PWM1 set to 0x30, Both set to 0x34
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x04, TWL_INTBR_PMBR1);
        //PWM0 Enable set to 0x05, PWM1 Enable set to 0x0a, Both set to 0x0f
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x05, TWL_INTBR_GPBR1);

	backlight_intensity = ((125 * (intensity)) / 100) + 2;
	twl_i2c_write_u8(TWL4030_MODULE_PWM0, 0x1, TWL_PWM_ON);
	twl_i2c_write_u8(TWL4030_MODULE_PWM0, backlight_intensity , TWL_PWM_OFF);

#endif
}

static void __init tao3530_lcd_display_init(void)
{
	gpio_request(TAO3530_LCD_PANEL_ENVDD, "lcd_panel_envdd");
	gpio_request(TAO3530_LCD_PON, "lcd_panel_pon");
	gpio_request(TAO3530_LCD_ENBKL, "LCD_ENBKL");

	gpio_direction_output(TAO3530_LCD_PON, 1); 
	gpio_direction_output(TAO3530_LCD_PANEL_ENVDD, 0);
	gpio_direction_output(TAO3530_LCD_ENBKL, 1);

	return;
}

static int tao3530_enable_lcd(struct omap_dss_device *dssdev)
{
	
	if (dvi_enabled) {
		printk(KERN_ERR "cannot enable LCD, DVI is enabled\n");
		return -EINVAL;
	}

	tao3530_set_backlight_intensity(tao3530_backlight_level);

#if defined(CONFIG_LVDS_18BIT)
        gpio_set_value(TAO3530_LCD_ENBKL, 1);
	gpio_set_value(TAO3530_LCD_PON, 1);
#else
        gpio_set_value(TAO3530_LCD_PANEL_ENVDD, 0);
        gpio_set_value(TAO3530_LCD_PON, 1);
#endif

	lcd_enabled = 1;
	return 0;
}

/*Disable LCD*/
static void tao3530_disable_lcd(struct omap_dss_device *dssdev)
{
#if defined(CONFIG_LVDS_18BIT)

        /*PWM0 Enable set to 0x04, Enable PWM1 set to 0x30, Both set to 0x34 */
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x34, TWL_INTBR_PMBR1);
        /*PWM0 Enable set to 0x05, PWM1 Enable set to 0x0a, Both set to 0x0f */
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x0f, TWL_INTBR_GPBR1);

	#if defined(CONFIG_PANEL_LITEMAX_DLF1095)/*Higher Duty Cycle with Lower Luminance*/
	twl_i2c_write_u8(TWL4030_MODULE_PWM0, 0x01, TWL_PWM_ON);
	twl_i2c_write_u8(TWL4030_MODULE_PWM0, 0x0 , TWL_PWM_OFF);
	#else
        twl_i2c_write_u8(TWL4030_MODULE_PWM1, 0x01, TWL_PWM_ON);
        twl_i2c_write_u8(TWL4030_MODULE_PWM1, 0x0 , TWL_PWM_OFF);
	#endif

	gpio_set_value(TAO3530_LCD_ENBKL, 0);
	gpio_set_value(TAO3530_LCD_PON, 0);

#else
        gpio_set_value(TAO3530_LCD_PANEL_ENVDD, 1);
        gpio_set_value(TAO3530_LCD_PON, 0);
	/*For Lg_lb043wq2 and Innolux_at070tn93, backlight off by PWM set to 0 */
        twl_i2c_write_u8(TWL4030_MODULE_PWM0, 0x01, TWL_PWM_ON);
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x04, TWL_INTBR_PMBR1);
        twl_i2c_write_u8(TWL4030_MODULE_PWM0, 0x0 , TWL_PWM_OFF);
        twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x05, TWL_INTBR_GPBR1);

#endif

	lcd_enabled = 0;
}

static struct omap_dss_device tao3530_lcd_device = {
	.name			= "lcd",
	.driver_name	= "panel-tao-series",
	.type			= OMAP_DISPLAY_TYPE_DPI,
#if defined(CONFIG_LVDS_18BIT)
	.phy.dpi.data_lines     = 18,
#else
	.phy.dpi.data_lines	= 24,
#endif
	.platform_enable	= tao3530_enable_lcd,
	.platform_disable	= tao3530_disable_lcd,

};
#else
static inline void __init tao3530_lcd_display_init(void) {return;}
#endif/*defined(TAO3530_LCD_ENABLED)*/

#if defined(TAO3530_TV_OUTPUT_ENABLED)
static int tao3530_enable_tv(struct omap_dss_device *dssdev)
{
	return 0;
}

static void tao3530_disable_tv(struct omap_dss_device *dssdev)
{
}

static struct omap_dss_device tao3530_tv_device = {
	.name			= "tv",
	.driver_name		= "venc",
	.type			= OMAP_DISPLAY_TYPE_VENC,
#if defined(CONFIG_OMAP2_VENC_OUT_TYPE_SVIDEO)
	.phy.venc.type		= OMAP_DSS_VENC_TYPE_SVIDEO,
#elif defined(CONFIG_OMAP2_VENC_OUT_TYPE_COMPOSITE)
	.phy.venc.type		= OMAP_DSS_VENC_TYPE_COMPOSITE,
#endif
	.platform_enable	= tao3530_enable_tv,
	.platform_disable	= tao3530_disable_tv,
};
#endif/*defined(TAO3530_TV_OUTPUT_ENABLED)*/

#if defined(TAO3530_DVI_ENABLED)
#define TAO3530_DVI_PANEL_EN_GPIO 199 /* GPIO 7 on TWL4030 */
static int tao3530_enable_dvi(struct omap_dss_device *dssdev)
{
	if (lcd_enabled) {
		printk(KERN_ERR "cannot enable DVI, LCD is enabled\n");
		return -EINVAL;
	}

#if defined(CONFIG_LVDS_18BIT)
	/*Re-program TWL_GPIO7 as GPIO*/
	//PWM0 Enable set to 0x04, Enable PWM1 set to 0x30, Both set to 0x34
	twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x04, TWL_INTBR_PMBR1);
	//PWM0 Enable set to 0x05, PWM1 Enable set to 0x0a, Both set to 0x0f
	twl_i2c_write_u8(TWL4030_MODULE_INTBR, 0x05, TWL_INTBR_GPBR1);
#endif
	gpio_set_value(TAO3530_DVI_PANEL_EN_GPIO, 1);
	dvi_enabled = 1;
	return 0;
}

static void tao3530_disable_dvi(struct omap_dss_device *dssdev)
{
	gpio_set_value(TAO3530_DVI_PANEL_EN_GPIO, 0);
	dvi_enabled = 0;
}

static struct omap_dss_device tao3530_dvi_device = {
	.name			= "dvi",
	.driver_name		= "generic_panel",
	.type			= OMAP_DISPLAY_TYPE_DPI,
	.phy.dpi.data_lines	= 24,
	.platform_enable	= tao3530_enable_dvi,
	.platform_disable	= tao3530_disable_dvi,
};
#endif/*defined(TAO3530_DVI_ENABLED)*/

static struct omap_dss_device *tao3530_dss_devices[] = {
#if defined(TAO3530_LCD_ENABLED)
	&tao3530_lcd_device,
#endif
#if defined(TAO3530_DVI_ENABLED)
	&tao3530_dvi_device,
#endif
#if defined(TAO3530_TV_OUTPUT_ENABLED)
	&tao3530_tv_device,
#endif
};

static struct omap_dss_board_info tao3530_dss_data = {
	.num_devices	= ARRAY_SIZE(tao3530_dss_devices),
	.devices	= tao3530_dss_devices,
#if defined(TAO3530_LCD_ENABLED)
	.default_device = &tao3530_lcd_device,
#elif defined(TAO3530_TV_OUTPUT_ENABLED)
	.default_device = &tao3530_tv_device,
#elif defined(TAO3530_DVI_ENABLED)
	.default_device = &tao3530_dvi_device,
#endif
};

static struct platform_device tao3530_dss_device = {
	.name		= "omapdss",
	.id		= -1,
	.dev		= {
		.platform_data = &tao3530_dss_data,
	},
};

#if defined(TAO3530_LCD_ENABLED)
static struct generic_bl_info tao3530_bl_platform_data = {
	.name			= "omap3evm-bklight",
	.max_intensity		= 100,
	.default_intensity	= DEFAULT_BACKLIGHT_LEVEL,
	.limit_mask		= 0,
	.set_bl_intensity	= tao3530_set_backlight_intensity,
	.kick_battery		= NULL,
};

static struct platform_device tao3530_bklight_device = {
	.name		= "generic-bl",
	.id		= -1,
	.dev		= {
		.parent		= &tao3530_dss_device.dev,
		.platform_data	= &tao3530_bl_platform_data,
	},
};
#endif

/*OAMP3 Flahs Init*/
static struct mtd_partition omap3tao3530_nand_partitions[] = {
	/* All the partition sizes are listed in terms of NAND block size */
	{
		.name		= "X-Loader",
		.offset		= 0,
		.size		= 4 * NAND_BLOCK_SIZE,
		.mask_flags	= MTD_WRITEABLE,	/* force read-only */
	},
	{
		.name		= "U-Boot",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x80000 */
		.size		= 15 * NAND_BLOCK_SIZE,
		.mask_flags	= MTD_WRITEABLE,	/* force read-only */
	},
	{
		.name		= "U-Boot Env",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x260000 */
		.size		= 1 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "Kernel",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x280000 */
		.size		= 32 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "File System",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x680000 */
		.size		= MTDPART_SIZ_FULL,
	},
};

static struct omap_nand_platform_data omap3tao3530_nand_data = {
	.options	= NAND_BUSWIDTH_16,
	.parts		= omap3tao3530_nand_partitions,
	.nr_parts	= ARRAY_SIZE(omap3tao3530_nand_partitions),
	.dma_channel	= -1,		/* disable DMA in OMAP NAND driver */
	.nand_setup	= NULL,
	.dev_ready	= NULL,
};

static struct resource omap3tao3530_nand_resource = {
	.flags		= IORESOURCE_MEM,
};

static struct platform_device omap3tao3530_nand_device = {
	.name		= "omap2-nand",
	.id		= -1,
	.dev		= {
		.platform_data	= &omap3tao3530_nand_data,
	},
	.num_resources	= 1,
	.resource	= &omap3tao3530_nand_resource,
};

/*UASRT*/
static struct omap_uart_config omap3_tao3530_uart_config __initdata = {
	.enabled_uarts	= ((1 << 0) | (1 << 1) | (1 << 2)),
};

/*PWM*/
static struct regulator_consumer_supply tao3530_vmmc1_supply = {
	.supply			= "vmmc",
};

static struct regulator_consumer_supply tao3530_vmmc2_supply = {
        .supply                 = "vmmc",
};

static struct regulator_consumer_supply tao3530_vsim_supply = {
	.supply			= "vmmc_aux",
};

static struct gpio_led gpio_leds[];

/* VMMC1 for MMC1 pins CMD, CLK, DAT0..DAT3 (20 mA, plus card == max 220 mA) */
static struct regulator_init_data tao3530_vmmc1 = {
	.constraints = {
		.min_uV			= 1850000,
		.max_uV			= 3150000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_VOLTAGE
					| REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &tao3530_vmmc1_supply,
};

static struct regulator_init_data tao3530_vmmc2 = {
	.constraints = {
		.min_uV                 = 1850000,
		.max_uV			= 3150000, 
		.apply_uV               = true,
		.valid_modes_mask       = REGULATOR_MODE_NORMAL
                                        | REGULATOR_MODE_STANDBY,
		.valid_ops_mask         = REGULATOR_CHANGE_MODE
                                        | REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies  = 1,
	.consumer_supplies      = &tao3530_vmmc2_supply,
};

/* VSIM for MMC1 pins DAT4..DAT7 (2 mA, plus card == max 50 mA) */
static struct regulator_init_data tao3530_vsim = {
	.constraints = {
		.min_uV			= 1800000,
		.max_uV			= 3000000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_VOLTAGE
					| REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &tao3530_vsim_supply,
};

static struct twl4030_hsmmc_info mmc[] = {
	{
		.mmc		= 1,
		.wires		= 8,
		.gpio_cd	= -EINVAL,
	#if defined(TAO3530_SD_WRITE_PROTECT_OFF)
		.gpio_wp	= -EINVAL,
	#else
		.gpio_wp        = 23,
	#endif
	},
	{
		.mmc		= 2,
		.wires		= 4,
		.gpio_cd	= -EINVAL,
		.gpio_wp	= -EINVAL,
		.transceiver    = true,
		.ocr_mask       = 0x00100000,//3.3v
	},
	{}	/* Terminator */
};

static struct gpio_led gpio_leds[] = {
	{
		.name			= "tao3530board::usr0",
		.default_trigger        = "heartbeat",
		.gpio			= -EINVAL,	/* gets replaced */
		.active_low		= true,
	},
};

static struct gpio_led_platform_data gpio_led_info = {
	.leds		= gpio_leds,
	.num_leds	= ARRAY_SIZE(gpio_leds),
};

static struct platform_device leds_gpio = {
	.name	= "leds-gpio",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_led_info,
	},
};

static int tao3530_twl_gpio_setup(struct device *dev,
		unsigned gpio, unsigned ngpio)
{
	/* gpio + 0 is "mmc0_cd" (input/IRQ) */
	omap_mux_init_gpio(23, OMAP_PIN_INPUT);
	mmc[0].gpio_cd = gpio + 0;
	twl4030_mmc_init(mmc);

	/* link regulators to MMC adapters */
	tao3530_vmmc1_supply.dev = mmc[0].dev; 
	tao3530_vmmc2_supply.dev = mmc[1].dev;
	tao3530_vsim_supply.dev = mmc[0].dev;

	/* the LED marked LED 2 (LEDA) */
	gpio_leds[0].gpio = gpio + TWL4030_GPIO_MAX + 1;

	/* gpio + 7 == DVI Enable */
	gpio_request(gpio + 7, "EN_DVI");
	gpio_direction_output(gpio + 7, 0);
	return 0;
}

static struct twl4030_gpio_platform_data tao3530_gpio_data = {
	.gpio_base	= OMAP_MAX_GPIO_LINES,
	.irq_base	= TWL4030_GPIO_IRQ_BASE,
	.irq_end	= TWL4030_GPIO_IRQ_END,
	.use_leds	= false,
	.pullups	= BIT(1),
	.pulldowns	= BIT(2) | BIT(6) | BIT(7) | BIT(8) | BIT(13)
				| BIT(15) | BIT(16) | BIT(17),
	.setup		= tao3530_twl_gpio_setup,
};

static struct platform_device tao3530_cam_device = {
	.name		= "tao3530_cam",
	.id		= -1,
};

#if defined(CONFIG_KEYBOARD_GPIO)||defined(CONFIG_KEYBOARD_GPIO_MODULE)
static struct gpio_keys_button gpio_buttons[] = {
	{
		.code			= BTN_EXTRA,
		.gpio			= 7,
		.desc			= "user",
		.wakeup			= 1,
	},
};

static struct gpio_keys_platform_data gpio_key_info = {
	.buttons	= gpio_buttons,
	.nbuttons	= ARRAY_SIZE(gpio_buttons),
};

static struct platform_device keys_gpio = {
	.name	= "gpio-keys",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_key_info,
	},
};
#endif

static struct twl4030_usb_data tao3530_usb_data = {
	.usb_mode	= T2_USB_MODE_ULPI,
};

/*AUDIO CODEC*/
static struct twl4030_codec_audio_data omap3tao3530_audio_data = {
	.audio_mclk = 26000000,
};

static struct twl4030_codec_data omap3tao3530_codec_data = {
	.audio_mclk = 26000000,
	.audio = &omap3tao3530_audio_data,
};

/*SVIDEO*/
static struct regulator_consumer_supply tao3530_vdda_dac_supply = {
        .supply         = "vdda_dac",
        .dev            = &tao3530_dss_device.dev,
};


/* VDAC for DSS driving S-Video (8 mA unloaded, max 65 mA) */
static struct regulator_init_data tao3530_vdac = {
    .constraints = {
        .min_uV         = 1800000,
        .max_uV         = 1800000,
        .apply_uV       = true,
        .valid_modes_mask   = REGULATOR_MODE_NORMAL
                    | REGULATOR_MODE_STANDBY,
        .valid_ops_mask     = REGULATOR_CHANGE_MODE
                    | REGULATOR_CHANGE_STATUS,
    },
    .num_consumer_supplies  = 1,
    .consumer_supplies  = &tao3530_vdda_dac_supply,
};

/* VPLL2 for digital video outputs */
#if 0
static struct regulator_consumer_supply tao3530_vdds_vdvi_supply = {
	.supply		= "vdvi",
	.dev		= &tao3530_dss_device.dev,
};

#else

/* VDDS_DSI for digital video outputs */
static struct regulator_consumer_supply tao3530_vdds_dsi_supply = {
        .supply         = "vdds_dsi",
        .dev            = &tao3530_dss_device.dev,
};
#endif


static struct regulator_init_data tao3530_vpll2 = {
    .constraints = {
        .name           = "VDVI",
        .min_uV         = 1800000,
        .max_uV         = 1800000,        
		.valid_modes_mask   = REGULATOR_MODE_NORMAL
                    | REGULATOR_MODE_STANDBY,
        .valid_ops_mask     = REGULATOR_CHANGE_MODE
                    | REGULATOR_CHANGE_STATUS,
    },
    .num_consumer_supplies  = 1,
#if 0
	.consumer_supplies  = &tao3530_vdds_vdvi_supply,
#else
	.consumer_supplies  = &tao3530_vdds_dsi_supply,
#endif
};

static struct twl4030_platform_data tao3530_twldata = {
	.irq_base	= TWL4030_IRQ_BASE,
	.irq_end	= TWL4030_IRQ_END,

	/* platform_data for children goes here */
	.usb		= &tao3530_usb_data,
	.gpio		= &tao3530_gpio_data,
	.codec		= &omap3tao3530_codec_data,
	.vmmc1		= &tao3530_vmmc1,
	.vmmc2      = &tao3530_vmmc2,
	.vsim		= &tao3530_vsim,
	.vdac       = &tao3530_vdac,
	.vpll2      = &tao3530_vpll2,
};

static struct i2c_board_info __initdata tao3530_i2c_1_boardinfo[] = {
	{
		I2C_BOARD_INFO("twl4030", 0x48),
		.flags = I2C_CLIENT_WAKE,
		.irq = INT_34XX_SYS_NIRQ,
		.platform_data = &tao3530_twldata,
	},
};

static struct i2c_board_info __initdata tao3530_i2c_2_boardinfo[] = {
#if defined(CONFIG_VIDEO_TVP514X) || defined(CONFIG_VIDEO_TVP514X_MODULE)
	{
		I2C_BOARD_INFO("tvp5146", 0x5D),
		.platform_data  = &tvp5146_pdata,
        },
#endif

	{},
};

static struct i2c_board_info __initdata tao3530_i2c_3_boardinfo[] = {
#if defined(CONFIG_RTC_DRV_S35390A)
        {
                I2C_BOARD_INFO("s35390a", 0x30),
                .type           = "s35390a",
		 },
#endif
	{},
};

static int __init omap3_tao3530_i2c_init(void)
{
	omap_register_i2c_bus(1, 2600, tao3530_i2c_1_boardinfo, ARRAY_SIZE(tao3530_i2c_1_boardinfo));
	omap_register_i2c_bus(2,  400, tao3530_i2c_2_boardinfo, ARRAY_SIZE(tao3530_i2c_2_boardinfo));//100 -> 400
	omap_register_i2c_bus(3,  400, tao3530_i2c_3_boardinfo, ARRAY_SIZE(tao3530_i2c_3_boardinfo));//100 -> 400
	return 0;
}

#if defined(TAO3530_TOUCHSCREEN_ENABLED) && \
	(defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE))

#define OMAP3_TAO3530_TS_GPIO	136

#include <plat/mcspi.h>
#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>

static struct omap2_mcspi_device_config ads7846_mcspi_config = {
	.turbo_mode	= 0,
	.single_channel	= 1,/* 0: slave, 1: master */
};

static int ads7846_get_pendown_state(void)
{
	return !gpio_get_value(OMAP3_TAO3530_TS_GPIO);
}

struct ads7846_platform_data ads7846_config = {
        .x_max              = 0x0fff,
        .y_max                  = 0x0fff,
        .x_plate_ohms       = TS_PLATE_OHMS,
        .pressure_max           = 255,
        .debounce_max           = TS_DEBOUNCE_MAX,
        .debounce_tol           = TS_DEBOUNCE_TOL,
        .debounce_rep           = TS_DEBOUNCE_REP,
        .get_pendown_state      = ads7846_get_pendown_state,
        .keep_vref_on           = 1,
        .settle_delay_usecs     = TS_SETTLE_DELAY,
        .wakeup                         = true,
};

struct spi_board_info omap3tao3530_spi_board_touch[] __initdata = {
	 {
		.modalias		= "ads7846",
		.bus_num		= 1,
		.chip_select		= 0,
		.max_speed_hz		= 1500000,
		.controller_data	= &ads7846_mcspi_config,
		.irq			= OMAP_GPIO_IRQ(OMAP3_TAO3530_TS_GPIO),
		.platform_data		= &ads7846_config,
	},
};

static void __init spi_touchscreen_init(void)
{
	if ((gpio_request(OMAP3_TAO3530_TS_GPIO, "ADS7846 pendown") == 0) &&
	    (gpio_direction_input(OMAP3_TAO3530_TS_GPIO) == 0)) {
		gpio_export(OMAP3_TAO3530_TS_GPIO, 0);
	} else {
		printk(KERN_ERR "could not obtain gpio for ADS7846_PENDOWN\n");
		return;
	}

	gpio_direction_input(OMAP3_TAO3530_TS_GPIO);

	omap_set_gpio_debounce(OMAP3_TAO3530_TS_GPIO, 1);
	omap_set_gpio_debounce_time(OMAP3_TAO3530_TS_GPIO, 0xa);

	spi_register_board_info(omap3tao3530_spi_board_touch,
			ARRAY_SIZE(omap3tao3530_spi_board_touch));
}

#else
static inline void __init spi_touchscreen_init(void) { return; }
#endif

#if defined(TAO3530_GSENSOR_ENABLED) && defined(CONFIG_INPUT_FF_MEMLESS)
/* Gsensor on SPI bus 4 */
/* TODO two interupts gpio 18 and 19 */
static struct spi_board_info omap3tao3530_spi_board_info[] = {
	{
		.modalias = "spidev",
		.max_speed_hz   = 1500000,
		.bus_num        = 4,
		.chip_select    = 0,
		.mode = SPI_MODE_0,
	},
};
static void __init spi_gsensor_dev_init(void)
{
	spi_register_board_info(omap3tao3530_spi_board_info,
		ARRAY_SIZE(omap3tao3530_spi_board_info));
}
#else
static inline void __init spi_gsensor_dev_init(void) { return; }
#endif

static struct omap_board_config_kernel omap3_tao3530_config[] __initdata = {
};

static void __init omap3_tao3530_init_irq(void)
{
	omap_board_config = omap3_tao3530_config;
	omap_board_config_size = ARRAY_SIZE(omap3_tao3530_config);
	if (cpu_is_omap3630()) {
		omap2_init_common_hw(mt46h32m32lf6_sdrc_params,
						mt46h32m32lf6_sdrc_params,
						_omap37x_mpu_rate_table,
						_omap37x_dsp_rate_table,
						_omap37x_l3_rate_table);
	} else {
		omap2_init_common_hw(mt46h32m32lf6_sdrc_params,
						mt46h32m32lf6_sdrc_params,
						_omap35x_mpu_rate_table,
						_omap35x_dsp_rate_table,
						_omap35x_l3_rate_table);
	}
	omap_init_irq();
	omap_gpio_init();
}

static struct platform_device *omap3_tao3530_devices[] __initdata = {
	&tao3530_dss_device,
	&leds_gpio,
#if defined(CONFIG_KEYBOARD_GPIO)||defined(CONFIG_KEYBOARD_GPIO_MODULE)
	&keys_gpio,
#endif
#if defined(TAO3530_LCD_ENABLED)
	&tao3530_bklight_device,
#endif
	&tao3530_cam_device,
};

static void __init omap3tao3530_flash_init(void)
{
	u8 cs = 0;
	u8 nandcs = GPMC_CS_NUM + 1;

	u32 gpmc_base_add = OMAP34XX_GPMC_VIRT;

	/* find out the chip-select on which NAND exists */
	while (cs < GPMC_CS_NUM) {
		u32 ret = 0;
		ret = gpmc_cs_read_reg(cs, GPMC_CS_CONFIG1);

		if ((ret & 0xC00) == 0x800) {
			printk(KERN_INFO "Found NAND on CS%d\n", cs);
			if (nandcs > GPMC_CS_NUM)
				nandcs = cs;
		}
		cs++;
	}

	if (nandcs > GPMC_CS_NUM) {
		printk(KERN_INFO "NAND: Unable to find configuration "
				 "in GPMC\n ");
		return;
	}

	if (nandcs < GPMC_CS_NUM) {
		omap3tao3530_nand_data.cs = nandcs;
		omap3tao3530_nand_data.gpmc_cs_baseaddr = (void *)
			(gpmc_base_add + GPMC_CS0_BASE + nandcs * GPMC_CS_SIZE);
		omap3tao3530_nand_data.gpmc_baseaddr = (void *) (gpmc_base_add);

		printk(KERN_INFO "Registering NAND on CS%d\n", nandcs);
		if (platform_device_register(&omap3tao3530_nand_device) < 0)
			printk(KERN_ERR "Unable to register NAND device\n");
	}
}

static struct ehci_hcd_omap_platform_data ehci_pdata __initconst = {
	//TODO: only set one PHY as enabled later.
        .port_mode[0] = EHCI_HCD_OMAP_MODE_UNKNOWN,
        .port_mode[1] = EHCI_HCD_OMAP_MODE_PHY,
        .port_mode[2] = EHCI_HCD_OMAP_MODE_UNKNOWN,

        .phy_reset  = false,
        .reset_gpio_port[0]  = -EINVAL,
        .reset_gpio_port[1]  = 162,  //note: we leave 162 for reset... but only if resister is removed .
        .reset_gpio_port[2]  = -EINVAL
};

#ifdef CONFIG_OMAP_MUX
static struct omap_board_mux board_mux[] __initdata = {
        /* Camera - Parallel Data */
        OMAP3_MUX(CAM_D0, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D1, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D2, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D3, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D4, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D5, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D6, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D7, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D8, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D9, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D10, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_D11, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_PCLK, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),

        /* Camera - HS/VS signals */
        OMAP3_MUX(CAM_HS, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),
        OMAP3_MUX(CAM_VS, OMAP_MUX_MODE0 | OMAP_PIN_INPUT),

        /* Camera - Reset GPIO 98 */
        OMAP3_MUX(CAM_FLD, OMAP_MUX_MODE4 | OMAP_PIN_OUTPUT),

        /* Camera - XCLK */
        OMAP3_MUX(CAM_XCLKA, OMAP_MUX_MODE0 | OMAP_PIN_OUTPUT),

		OMAP3_MUX(ETK_D6, OMAP_MUX_MODE4 | OMAP_PIN_OUTPUT),
		OMAP3_MUX(ETK_D8, OMAP_MUX_MODE4 | OMAP_PIN_OUTPUT),

#if defined(CONFIG_KEYBOARD_GPIO)||defined(CONFIG_KEYBOARD_GPIO_MODULE)
		OMAP3_MUX(SYS_BOOT5, OMAP_MUX_MODE4 | OMAP_PIN_INPUT),
#endif
	{ .reg_offset = OMAP_MUX_TERMINATOR },
};
#else
#define board_mux	NULL
#endif

static void __init omap3_tao3530_init(void)
{
	omap3_mux_init(board_mux, OMAP_PACKAGE_CBB);

	omap3_tao3530_i2c_init();

	platform_add_devices(omap3_tao3530_devices, ARRAY_SIZE(omap3_tao3530_devices));

	omap_serial_init();

	usb_musb_init();
	usb_ehci_init(&ehci_pdata);
	omap3tao3530_flash_init();

	spi_touchscreen_init();
	spi_gsensor_dev_init();

	tao3530_lcd_display_init();

	omap3tao3530_init_smsc911x();

        if ((gpio_request(22, "GPIO_22") == 0) &&
            (gpio_direction_output(22, 1) == 0)) {
                gpio_export(22, 0);
                gpio_set_value(22, 0);
        }

#if defined(TAO3530_SDIO_WIFI)
        if ((gpio_request(GPIO_WIFI_NRESET, "GPIO_WIFI_NRESET") == 0) &&
            (gpio_direction_output(GPIO_WIFI_NRESET, 1) == 0)) {
                gpio_export(GPIO_WIFI_NRESET, 0);
	#if defined(CONFIG_LIBERTAS_SDIO_MODULE)
                gpio_set_value(GPIO_WIFI_NRESET, 1);//Power Off by default, modules insert/remove will on/off the wifi
	#else
		gpio_set_value(GPIO_WIFI_NRESET, 0);//Power On by default, card detect pin will on/off the wifi
	#endif
        } else
                pr_warning("TAO3530 : Could not obtain gpio GPIO_WIFI_NRESET\n");
#endif

}

static void __init omap3_tao3530_map_io(void)
{
	omap2_set_globals_343x();
	omap2_map_common_io();
}

/*MACHINE_START(OMAP3_TAO3530, TAO3530_NAME_STRING)*/
MACHINE_START(OMAP3_TAO3530, "OMAP3 EVM")/*We mimic ourselves as omap3evm*/
	/* Maintainer: Edward Lin - Technexion: linuxfae@technexion.com */
	.phys_io	= 0x48000000,
	.io_pg_offst	= ((0xfa000000) >> 18) & 0xfffc,
	.boot_params	= 0x80000100,
	.map_io		= omap3_tao3530_map_io,
	.init_irq	= omap3_tao3530_init_irq,
	.init_machine	= omap3_tao3530_init,
	.timer		= &omap_timer,
MACHINE_END
