
#include <linux/module.h>
#include <linux/delay.h>

#include <plat/display.h>

struct tao_series_panels {
        const char *name;
	struct omap_video_timings panel_timings;
};

static const struct tao_series_panels tao_series_paneldb[] = 
{
	{"litemax_dlf1095_enn_a01",	{1024, 768, 65000, 5, 50, 265, 2, 16, 20}},
	{"hannstar_hsd101pfw1_a00",	{1024, 600, 45000, 5, 43, 104, 5, 20, 24}},
	{"auo_b089aw01",		{1024, 600, 54000, 42, 48, 320, 5, 5, 20}},
	{"lg_lb043wq2",			{480, 272, 9000, 42, 3, 2, 11, 3, 2}},
	{"innolux_at070tn94",		{800, 480, 33300, 1, 210, 46, 1, 22, 23}},
};

#if defined(CONFIG_PANEL_LITEMAX_DLF1095)//This is for LITEMAX DLF1095-ENN-A01
#define TAO_SERIES_PANEL_NAME "LITEMAX DLF1095-ENN-A01"
static struct omap_video_timings tao_series_timings = {
        .x_res          = 1024,
        .y_res          = 768,
        .pixel_clock    = 65000,
        .hfp            = 50,
        .hsw            = 5,
        .hbp            = 265,
        .vfp            = 16,
        .vsw            = 2,
        .vbp            = 20,
};
#endif

#if defined(CONFIG_PANEL_HANNSTAR_HSD101PFW1)//This is for HannStar HSD101PFW1-A00
#define TAO_SERIES_PANEL_NAME "HannStar HSD101PFW1-A00"
static struct omap_video_timings tao_series_timings = {
        .x_res          = 1024,
        .y_res          = 600,
        .pixel_clock    = 45000,
        .hfp            = 43,
        .hsw            = 5,
        .hbp            = 104,
        .vfp            = 20,
        .vsw            = 5,
        .vbp            = 24,
};
#endif

#if defined(CONFIG_PANEL_AUO_B089AW01)//This is for AUO B089AW01
#define TAO_SERIES_PANEL_NAME "AUO B089AW01"
static struct omap_video_timings tao_series_timings = {
        .x_res          = 1024,
        .y_res          = 600,
        .pixel_clock    = 54000,
        .hfp            = 48,
        .hsw            = 42,
        .hbp            = 320,
        .vfp            = 5,
        .vsw            = 5,
        .vbp            = 20,
};
#endif

#if defined(CONFIG_PANEL_LG_LB043WQ2)//This is for LG LB043WQ2
#define TAO_SERIES_PANEL_NAME "LG LB043WQ2"
static struct omap_video_timings tao_series_timings = {
	.x_res = 480,
	.y_res = 272,
#if 0
	.pixel_clock	= 8000,
	.hsw            = 41,
	.hfp            = 8,
	.hbp            = 45-41,
	.vsw            = 10,
	.vfp            = 4,
	.vbp            = 12-10,
#else
	.pixel_clock	= 9000,
	.hsw		= 42,
	.hfp		= 3,
	.hbp		= 2,
	.vsw		= 11,
	.vfp		= 3,
	.vbp		= 2,
#endif

};

#endif

#if defined(CONFIG_PANEL_INNOLUX_AT070TN93)//This is for Innolux AT070TN94/AT070TN94
#define TAO_SERIES_PANEL_NAME "Innolux AT070TN94/AT070TN94"
static struct omap_video_timings tao_series_timings = {
        .x_res          = 800,
        .y_res          = 480,
        .pixel_clock    = 33300,
        .hfp            = 210,
        .hsw            = 1,
        .hbp            = 46,
        .vfp            = 22,
        .vsw            = 1,
        .vbp            = 23,
};

#endif

static char *disp_timings;
module_param_named(disp_timings, disp_timings, charp, 0);

static u16 data_lines = 24;
static void set_tao_series_panel_timings(void)
{
	struct omap_video_timings t;
	if(sscanf(disp_timings, "%u,%hu/%hu/%hu/%hu,%hu/%hu/%hu/%hu,%hu",
                                &t.pixel_clock,
                                &t.x_res, &t.hfp, &t.hbp, &t.hsw,
                                &t.y_res, &t.vfp, &t.vbp, &t.vsw,
				&data_lines) != 10)
 	{
		printk(KERN_INFO "Display timing settings are incorrect\n");
		return;
	}
	tao_series_timings = t;
}

static int tao_series_panel_probe(struct omap_dss_device *dssdev)
{
	if(disp_timings)
		set_tao_series_panel_timings();

	dssdev->panel.config = OMAP_DSS_LCD_TFT | OMAP_DSS_LCD_IVS |
		OMAP_DSS_LCD_IHS;
	
	dssdev->panel.timings = tao_series_timings;
	printk(KERN_ERR "%s Probe\n",TAO_SERIES_PANEL_NAME);
	return 0;
}

static void tao_series_panel_remove(struct omap_dss_device *dssdev)
{
}

static int tao_series_panel_enable(struct omap_dss_device *dssdev)
{
	int r = 0;

	/* wait couple of vsyncs until enabling the LCD */
	msleep(50);

	if (dssdev->platform_enable)
		r = dssdev->platform_enable(dssdev);

	return r;
}

static void tao_series_panel_disable(struct omap_dss_device *dssdev)
{
	if (dssdev->platform_disable)
		dssdev->platform_disable(dssdev);

	/* wait at least 5 vsyncs after disabling the LCD */

	msleep(100);
}

static int tao_series_panel_suspend(struct omap_dss_device *dssdev)
{
	tao_series_panel_disable(dssdev);
	return 0;
}

static int tao_series_panel_resume(struct omap_dss_device *dssdev)
{
	return tao_series_panel_enable(dssdev);
}

static struct omap_dss_driver tao_series_driver = {
	.probe		= tao_series_panel_probe,
	.remove		= tao_series_panel_remove,

	.enable		= tao_series_panel_enable,
	.disable	= tao_series_panel_disable,
	.suspend	= tao_series_panel_suspend,
	.resume		= tao_series_panel_resume,

	.driver         = {
		.name	= "panel-tao-series",
		.owner  = THIS_MODULE,
	},
};

static int __init tao_series_panel_drv_init(void)
{
	return omap_dss_register_driver(&tao_series_driver);
}

static void __exit tao_series_panel_drv_exit(void)
{
	omap_dss_unregister_driver(&tao_series_driver);
}

module_init(tao_series_panel_drv_init);
module_exit(tao_series_panel_drv_exit);
MODULE_LICENSE("GPL");
